/* Copyright (C) 2023  José Luis Rubio Leira

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>
*/

//---- Importaciónes
const express=require('express');  //Servidor web express
const app = express();
const winston=require('winston'); //winston (Sistema de logging)
let Client = require('ssh2-sftp-client'); //Cliente ssh-sftp
const fs=require('fs-extra-promise');  //Manejo de ficheros
const moment=require('moment-timezone');  //Gestión de zonas horarias
require('./startup/logging')();
require('./startup/db')(app);
const { DateTime } = require("luxon"); //Gestión de fechas
const { ConsoleTransportOptions } = require('winston/lib/winston/transports');
const {Order,validate}=require('./models/order');  //Modelo de pedidos

//Definimos la zona horaria
moment().tz('Europe/Madrid').format();
process.env.TZ='Europe/Madrid';

//Definimos rutas de entrada de pedidos y de pedidos procesados
const remote_route_in='/datos/in'
const remote_route_out='/datos/out'

var intervalId=0; 
var ejecutando=0;

//Cargamos los centros 
const { Centro } = require('./models/centro');

//Configuramos el cliente SFTP
let sftp = new Client();
let conn;

//Definimos los parámetros de la conexión con el servidor de eroski
const connection={
    host: 'localhost',
    port: '22',
    username: 'i1567',
    tryKeyboard: true
}

//Establecemos el inicio de sesion simulando introducción interactiva por teclado de credenciales
sftp.on('keyboard-interactive', (name, instructions, instructionsLang, prompts, finish) => { finish(['pruebas']); });

//Función downloadFiles:  Función que se llama en cada intervalo y que comprueba si existen ficheros nuevos, y 
//si existen, los descarga del servidor, los procesa y los almacena en la base de datos
async function downloadFiles(){
    let s=undefined;
    let r=undefined;
    let q=undefined;

    //Intentamos establecer conexión con el servidor de Eroski
    try{
        await sftp.connect(connection);
        console.log('Conectado al Servidor FTP de eroski...')
    }
    catch(err){ console.log(err, 'catch error'); }

    //Procesado de ficheros
    try{
        console.log('Verificando si existen ficheros nuevos...')
        //Obtenemos la lista de ficheros de la ruta de entrada de pedidos
        const files=await sftp.list(remote_route_in);
        for(i in files) {  //Si existen ficheros se procesan...
            const stats=await sftp.stat(remote_route_in+'/'+files[i].name);
            if(stats.isFile){   //Se comprueba para cada entrada de la lista si el objeto es un fichero
                if (intervalId==0){  //Si el intervalo se establece a 0, se finaliza la conexión
                    await sftp.end();
                    ejecutando=0;
                    return;
                } 
                //Obtenemos el fichero, descargandolo en la ruta local
                try{
                    s=await sftp.fastGet(remote_route_in+'/'+files[i].name,'./'+files[i].name);
                    console.log(s);
                }
                catch(error){
                    console.log('Algo ha fallado descargando el fichero '+files[i].name+': '+error.message);
                    await sftp.end();
                    ejecutando=0;
                    return;
                }
                                
                //Leemos el fichero y lo cargamos en la bbdd
                try{
                    //Lectura del fichero 
                    const data=await fs.readFileAsync('./'+files[i].name,'latin1');
                    const dat=String(data);

                    //Construimos los campos de fechas (entrega y generación de ticket)
                    const st_fecha_entrega_desde='20'+dat.substr(15,2)+'-'+dat.substr(17,2)+'-'+dat.substr(19,2)+'T'+dat.substr(21,2)+':'+dat.substr(23,2)+':00+0100';
                    const st_fecha_entrega_hasta='20'+dat.substr(15,2)+'-'+dat.substr(17,2)+'-'+dat.substr(19,2)+'T'+dat.substr(25,2)+':'+dat.substr(27,2)+':00+0100';
                    const st_fecha_generacion_ticket='20'+dat.substr(29,2)+'-'+dat.substr(31,2)+'-'+dat.substr(33,2)+'T'+dat.substr(35,2)+':'+dat.substr(37,2)+':00+0100';

                    const dt_fecha_entrega_desde=DateTime.fromISO(st_fecha_entrega_desde,{setZone: true, zone:'Europe/Madrid'});
                    const dt_fecha_entrega_hasta=DateTime.fromISO(st_fecha_entrega_hasta,{setZone: true, zone:'Europe/Madrid'});
                    const dt_fecha_generacion_ticket=DateTime.fromISO(st_fecha_generacion_ticket,{setZone: true, zone:'Europe/Madrid'});


                    const cod=dat.substr(1,4);
                    const reparto=dat.substr(5,10).trimRight();

                    //Validaciónes de centro correcto y de pedidos repetidos
                    const cent=await Centro.findOne({cod_centro: cod});
                    if (!cent){
                        winston.error('Error. Centro '+cod+'no existe');
                        continue;
                    }
                    const rep=await Order.findOne({nombre_fichero: files[i].name});
                    if (rep){
                        winston.error('Error. El pedido '+files[i].name+' ya existe');
                        continue;
                    }

                    //Cargamos los datos del fichero en el objeto pedido    
                    let pedido={
                        tipo_movimiento: dat.substr(0,1),
                        centro:cent,
                        id_reparto:reparto,
                        fecha_entrega_desde: dt_fecha_entrega_desde.toJSDate(),
                        fecha_entrega_hasta: dt_fecha_entrega_hasta.toJSDate(),
                        fecha_generacion_ticket: dt_fecha_generacion_ticket.toJSDate(),
                        nombre_cliente: dat.substr(39,50).trimRight(),
                        apellidos_cliente: dat.substr(89,100).trimRight(),
                        tipo_via_nombre: dat.substr(189,145).trimRight(),
                        portal_num_esc_piso_mano: dat.substr(334,45),
                        codigo_postal: dat.substr(379,5),
                        localidad_municipio: dat.substr(384,100).trimRight(),
                        telefono_cliente: dat.substr(484,9).trimRight(),
                        identificativo_cliente: dat.substr(493,9),
                        ascensor_cliente: dat.substr(502,1),
                        info_adicional: dat.substr(503,50),
                        cajas_seco: parseInt(dat.substr(553,2)),
                        cajas_fresco: parseInt(dat.substr(555,2)),
                        cajas_congelado: parseInt(dat.substr(557,2)),
                        bolsas_fresco: parseInt(dat.substr(559,2)),
                        bolsas_congelado: parseInt(dat.substr(561,2)),
                        importe_ticket: parseInt(dat.substr(563,10)) / 100,
                        tipo_cliente: dat.substr(573,1),
                        nombre_fichero: files[i].name,
                        fecha_carga: Date()
                    };
                    //Validamos y registramos el pedido en MongoDB
                    //1. Validación del registro
                    const {error}=validate(pedido);
                    if(error){
                        //Hacerlo en winston
                        winston.error('Error validando en pedido: '+error.details[0].message);
                        continue;
                    }
                    
                    //2. Grabación en MongoDB
                    let order=new Order(pedido);
                    order=await order.save();
                    console.log('Grabado pedido '+reparto+' de forma correcta.');
                    //console.log(order);
    
                }
                catch(error){
                    winston.error('Error leyendo fichero local: ./'+files[i].name);
                    sftp.end();
                    return;
                }
            
                //Copiamos el fichero a la ruta out remota (pedidos procesados)...
                try{
                    r=await sftp.rcopy(remote_route_in+'/'+files[i].name,remote_route_out+'/'+files[i].name);
                    q=await sftp.delete(remote_route_in+'/'+files[i].name);
                    console.log(q);
                }
                catch(error){
                    winston.error('Algo ha fallado cuando se movía el fichero a la carpeta /out: '+error);
                    if(error.message.indexOf("Destino ya existe") > 0){
                        try{
                            q=await sftp.delete(remote_route_in+'/'+files[i].name);
                        }
                        catch(err){
                            winston.error('Algo DEFINITIVAMENTE ha fallado cuando se movía el fichero a la carpeta /out: '+error);
                            sftp.end();
                            return;
                        }
                        
                    }
                    sftp.end();
                    return;
                }

                //Si todo ha ido bien, borramos el fichero local descargado
                fs.rm('./'+files[i].name,{maxRetries:10,retryDelay:500},(error)=>{
                    if(error){
                        winston.error('Error eliminando el fichero local: ',error);
                        sftp.end();
                        return;
                    }
                    else{
                        console.log('Fichero local eliminado correctamente.')
                    }
                });           
            }
                
        }
    }
    catch(err){
        winston.error('Error grave: '+err);
    }

    //Finalizamos la conexión hasta el siguiente intervalo
    try{ await sftp.end();
        console.log('Esperando 1 minuto para conectarse de nuevo y comprobar cambios...')
        ejecutando=0;        
    }
    catch(err){ console.log(err, 'catch error');}
}

//La API route /start hace que se establezca el intervalo (Por defecto 60 segundos), y se comienze la descarga y el procesado de ficheros
app.get('/start',(req,res)=>{
    if (intervalId!=0) return res.send("La sesión está ya iniciada...") ;
    res.send('Iniciado proceso de ficheros...');
    console.log('Comenzando sesión de descarga de ficheros...')
    downloadFiles();
    intervalId=setInterval(()=>{
        if (ejecutando===0){
            ejecutando=1;
            downloadFiles();
        }
    }, 60000);
});

//La API route /stop finaliza la sesión de descarga y procesado de ficheros.
app.get('/stop',(req,res)=>{
    if(intervalId == 0) return res.send('No existe sesión que finalizar.');
    console.log('Finalizando Sesión...')        
    clearInterval(intervalId);
    intervalId=0;
    return res.send('Finalizado.');    
});


//Establecemos la escucha en el puerto 3000
const port = process.env.PORT || 3000;
app.listen(port,()=>{
    console.log(`Escuchando en el puerto ${port}...`)
})

