<?php

namespace Tests\Unit;

use App\Http\Controllers\OrderController;
use App\Models\Order;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Tests\TestCase;
use App\Models\User;

class OrderControllerTest extends TestCase
{
    
    public function testOrdersFecha()
    {
        $request = new Request();
        $request->setMethod('POST');
        $request->request->add(['fecha' => '2023-05-23']);

        $controller = new OrderController();

        // Acción
        $response = $controller->ordersFecha($request);

        // Comprobaciones
        $this->assertEquals('pedidos', $response->getName());   //Comprobamos que devuelve una vista de nombre 'pedidos'
        $this->assertArrayHasKey('orders', $response->getData());  //Comprobamos que se devuelve un array de orders
        $this->assertArrayHasKey('fecha', $response->getData());  //Comprobamos que se devuelve la fecha que se consulta

        $datos = $response->getData()['orders'];
        $this->assertGreaterThan(0, $datos->count());   //Comprobamos que se devuelven pedidos

        foreach($datos as $order){
            //Comprobamos que para cada pedido obtenido, la fecha sea la correcta
            $this->assertEquals($response->getData()['fecha'], $order->fecha_entrega_desde->toDateTime()->setTimezone(new \DateTimeZone('Europe/Madrid'))->format('Y-m-d'));            
            
        }

    }


    public function testIndex()
    {
        $request = new Request();
        $request->setMethod('POST');
        $request->request->add(['fec' => '2023-05-23']);

        $controller = new OrderController();

        // Crear un usuario ficticio.
        $user = User::factory()->create();

        // Simular que el usuario está autenticado.
        $this->actingAs($user);        

        // Acción
        $response = $controller->index($request);

        // Comprobaciones
        $this->assertEquals('pedidos', $response->getName());   //Comprobamos que devuelve una vista de nombre 'pedidos'
        $this->assertArrayHasKey('orders', $response->getData());  //Comprobamos que se devuelve un array de orders
        $this->assertArrayHasKey('fecha', $response->getData());  //Comprobamos que se devuelve la fecha que se consulta

        $datos = $response->getData()['orders'];
        $this->assertGreaterThan(0, $datos->count());   //Comprobamos que se devuelven pedidos

        foreach($datos as $order){
            //$this->assertEquals($response->getData()['fecha'], $order->fecha_entrega_desde );
            $this->assertEquals($response->getData()['fecha'], $order->fecha_entrega_desde->toDateTime()->setTimezone(new \DateTimeZone('Europe/Madrid'))->format('Y-m-d'));            
            
        }

    }

    public function testActualizarEstado()
    {

        //Eliminamos todos los documentos que tengan un campo fecha = '01/01/2000' (El campo fecha no se utiliza en la aplicacion. Las fechas tienen otros nombres de campo)
        $order=Order::where('fecha','01/01/2000')->delete();       

        // Crear un usuario ficticio.
        $user = User::factory()->create();
        $user->email='jlrubiol@gmail.com';
        $this->actingAs($user);

        // Crear un pedido ficticio.
        $order = new Order([
            'centro'=>'4041',
            'fecha' =>'01/01/2000',
            'estado' => 0 // Estado inicial.
        ]);

        $order->save(); //Guarda los datos para el usuario

        // Simular que el usuario está autenticado.
        $this->actingAs($user);
        $fecha_actu=Carbon::now();

        // Enviar una solicitud POST a la función incluyendo los datos necesarios que pase de pendiente a entregado.
        $request = new Request();
        $request->setMethod('POST');
        $request->request->add(['id'=> $order->_id, 'centro' => $order->centro, 'fecha='> $fecha_actu,'estado' =>$order->estado]);        

        $controller = new OrderController();        
        $response=$controller->actualizarEstado($request);

        // Comprobar que el estado del pedido se ha actualizado correctamente de pendiente a entregado.
        $order->refresh(); //Refrescamos los datos del pedido
        $this->assertEquals(1, $order->estado);
        //Comprobamos que se ha actualizado correctamente el campo de usuario modificador
        $this->assertEquals($user->email, $order->usuario);
        //Comrprobamos que se ha actualizado la fecha de modificación.
        $this->assertEquals($fecha_actu->format('Y-m-d'), $order->updated_at->toDateTime()->setTimezone(new \DateTimeZone('Europe/Madrid'))->format('Y-m-d'));        


        // Enviar una solicitud POST a la función incluyendo los datos necesarios que pase de entregado a pendiente.
        $request = new Request();
        $request->setMethod('POST');
        $request->request->add(['id'=> $order->_id, 'centro' => $order->centro, 'fecha='> $fecha_actu,'estado' =>$order->estado]);        

        $controller = new OrderController();        
        $response=$controller->actualizarEstado($request);
        
        // Comprobar que el estado del pedido se ha actualizado correctamente de pendiente a entregado.
        $order->refresh(); //Refrescamos los datos del pedido
        $this->assertEquals(0, $order->estado);
        //Comprobamos que se ha actualizado correctamente el campo de usuario modificador
        $this->assertEquals($user->email, $order->usuario);
        //Comrprobamos que se ha actualizado la fecha de modificación.
        $this->assertEquals($fecha_actu->format('Y-m-d'), $order->updated_at->toDateTime()->setTimezone(new \DateTimeZone('Europe/Madrid'))->format('Y-m-d'));        

        //Eliminamos el pedido de prueba para sucesivos tests
        $order=Order::destroy($order->id);       
    }


}
